@@ -8,6 +8,7 @@
 #include <locale.h>
 #include <string.h>
 
+#define APP_SEARCH_HOME_PLUGIN_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), APP_SEARCH_TYPE_HOME_PLUGIN, AppSearchHomePluginPrivate))
 HD_DEFINE_PLUGIN_MODULE (AppSearchHomePlugin, app_search_home_plugin, HD_TYPE_HOME_PLUGIN_ITEM)
 
 #define CONFIG_PATH "/usr/share/applications/hildon/"
@@ -20,9 +21,59 @@ HD_DEFINE_PLUGIN_MODULE (AppSearchHomePlugin, app_search_home_plugin, HD_TYPE_HO
 #define DESKTOP_ENTRY_CONFIG_GROUP "Desktop Entry" 
 #define TEXTDOMAIN_KEY "X-Text-Domain"
 #define NAME_KEY "Name"
+#define COMMENT_KEY "Comment"
 #define ICON_KEY "Icon"
 #define NODISPLAY_KEY "NoDisplay"
 
+struct _AppSearchHomePluginPrivate
+{
+  GtkListStore* app_list;
+  GtkTreeModel* filtered_app_list;
+  GThread* init_app_list_thread;
+  GtkWidget* entry;
+};
+
+typedef struct _result_data_t
+{
+  AppSearchHomePlugin* desktop_plugin;
+  GdkPixbuf* icon;
+  gchar* app_display_name;
+  gchar* app_name;
+  gchar* app_comment;
+  gchar* desktop_file_name;
+} result_data_t;
+
+
+const gchar*
+_get_default_text_color()
+{
+  static gchar buf[40] = {0};
+  if(buf[0] == '\0')
+  {
+    GdkColor color;
+    GtkStyle *style = gtk_rc_get_style_by_paths(gtk_settings_get_default(),
+						NULL, NULL, GTK_TYPE_LABEL);
+    if(gtk_style_lookup_color(style, "DefaultTextColor", &color))
+      sprintf(buf, "#%02x%02x%02x", color.red / 256, color.green / 256, color.blue / 256);
+  }
+  return buf;
+}
+
+const gchar*
+_get_active_text_color()
+{
+  static gchar buf[40] = {0};
+  if(buf[0] == '\0')
+  {
+    GdkColor color;
+    GtkStyle *style = gtk_rc_get_style_by_paths(gtk_settings_get_default(),
+						NULL, NULL, GTK_TYPE_LABEL);
+    if(gtk_style_lookup_color(style, "ActiveTextColor", &color))
+      sprintf(buf, "#%02x%02x%02x", color.red / 256, color.green / 256, color.blue / 256);
+  }
+  return buf;
+}
+
 static
 gchar* 
 _retrieve_app_name(GKeyFile* key_file)
@@ -48,6 +99,35 @@ _retrieve_app_name(GKeyFile* key_file)
   return NULL;
 }
 
+static
+gchar* 
+_retrieve_app_comment(GKeyFile* key_file)
+{
+  if(g_key_file_has_key(key_file, DESKTOP_ENTRY_CONFIG_GROUP, TEXTDOMAIN_KEY, NULL) &&
+     g_key_file_has_key(key_file, DESKTOP_ENTRY_CONFIG_GROUP, COMMENT_KEY, NULL))
+  {
+    gchar* text_domain = g_key_file_get_string(key_file, DESKTOP_ENTRY_CONFIG_GROUP, TEXTDOMAIN_KEY, NULL);
+    gchar* app_comment_key = g_key_file_get_string(key_file, DESKTOP_ENTRY_CONFIG_GROUP, COMMENT_KEY, NULL);
+    gchar* app_comment = g_strdup(dgettext(text_domain, app_comment_key));
+    g_free(text_domain);
+    if(g_strcmp0(app_comment, app_comment_key) == 0)
+    {
+      g_free(app_comment);
+      app_comment = NULL;
+    }
+    g_free(app_comment_key);
+    return app_comment;
+  }
+  else if(g_key_file_has_key(key_file, DESKTOP_ENTRY_CONFIG_GROUP, COMMENT_KEY, NULL))
+  {
+    gchar* app_comment = g_key_file_get_string(key_file, DESKTOP_ENTRY_CONFIG_GROUP, COMMENT_KEY, NULL);
+    gchar* comment = g_strdup(dgettext(GETTEXT_PACKAGE, app_comment));
+    g_free(app_comment);
+    return comment;
+  }
+  return NULL;
+}
+
 static GdkPixbuf*
 _retrieve_icon(GKeyFile* key_file)
 {
@@ -93,9 +173,29 @@ _shouldnt_be_visible(GKeyFile* key_file)
   }
   return FALSE;
 }
+static gboolean
+_append_result_entry(gpointer user_data)
+{
+  result_data_t* result_data = (result_data_t*)user_data;
+  GtkTreeIter iter;
+  gtk_list_store_append(result_data->desktop_plugin->priv->app_list, &iter);
+  gtk_list_store_set(result_data->desktop_plugin->priv->app_list, &iter,
+		     0, result_data->icon,
+		     1, result_data->app_display_name,
+		     2, result_data->app_name,
+		     3, result_data->app_comment,
+		     4, result_data->desktop_file_name,
+		     -1);
+  g_free(result_data->app_display_name);
+  g_free(result_data->app_name);
+  g_free(result_data->app_comment);
+  g_free(result_data->desktop_file_name);
+  g_free(result_data);
+  return FALSE;
+}
 
 static void
-_add_app_from_desktop_file_to_liststore(const gchar* file_name, GtkListStore* list_store)
+_add_app_from_desktop_file_to_liststore(const gchar* file_name, AppSearchHomePlugin* desktop_plugin)
 {
   GKeyFile *key_file = g_key_file_new();
   if(g_key_file_load_from_file(key_file, file_name, G_KEY_FILE_KEEP_COMMENTS, NULL))
@@ -106,33 +206,50 @@ _add_app_from_desktop_file_to_liststore(const gchar* file_name, GtkListStore* li
     else
     {
       gchar* app_name = _retrieve_app_name(key_file);
+
       if(app_name == NULL)
       {
 	return;
       }
+      gchar* app_comment = _retrieve_app_comment(key_file);
       GdkPixbuf* icon = _retrieve_icon(key_file);
       gchar* desktop_file_name = g_path_get_basename(file_name);
-      GtkTreeIter iter;
-      gtk_list_store_append(list_store, &iter);
-      gtk_list_store_set(list_store, &iter,
-			 0, icon,
-			 1, app_name,
-			 2, desktop_file_name,
-			 -1);
+      result_data_t* result_data = g_new0(result_data_t, 1);
+      result_data->desktop_plugin = desktop_plugin;
+      result_data->icon = icon;
+      result_data->app_name = g_strdup(app_name);
+      if(app_comment == NULL)
+      {
+	result_data->app_display_name = 
+	  g_markup_printf_escaped("<span color=\"%s\">%s</span>", _get_default_text_color(), app_name);
+	result_data->app_comment = g_strdup("");
+      }
+      else
+      {
+	result_data->app_comment = g_strdup(app_comment);
+	result_data->app_display_name = 
+	  g_markup_printf_escaped("<span color=\"%s\">%s</span>\n"
+				  "<span size=\"x-small\" color=\"%s\">%s</span>", 
+				  _get_default_text_color(), app_name,
+				  _get_active_text_color(), app_comment);
+      }
+      result_data->desktop_file_name = g_strdup(desktop_file_name);
+      g_idle_add((GSourceFunc)_append_result_entry, result_data);
       g_free(desktop_file_name);
       g_free(app_name);
-      g_object_unref(icon);
+      g_free(app_comment);
+      //      g_object_unref(icon);
     }
     g_key_file_free(key_file);
   }
 }
 
-static void
-_fill_app_list_liststore_from_app_config_path(GtkListStore* list_store)
+static gpointer
+_fill_app_list_liststore_from_app_config_path(gpointer user_data)
 {
+  AppSearchHomePlugin* desktop_plugin = APP_SEARCH_HOME_PLUGIN(user_data);
   GError* error = NULL;
   GDir* directory = g_dir_open(CONFIG_PATH, 0, &error);
-
   if(error)
   {
     g_warning("error on open directory %s\n", CONFIG_PATH);
@@ -145,38 +262,53 @@ _fill_app_list_liststore_from_app_config_path(GtkListStore* list_store)
     while((file_name = g_dir_read_name(directory))!=NULL)
     {
       gchar* full_path = g_build_filename(CONFIG_PATH, file_name, NULL);
-      _add_app_from_desktop_file_to_liststore(full_path, list_store);
+      _add_app_from_desktop_file_to_liststore(full_path, desktop_plugin);
       g_free(full_path);
     }
     g_dir_close(directory);
   }
+  return NULL;
 }
 
-/*
-static gboolean 
-search_app_func(GtkTreeModel *model,
-		gint column,
-		const gchar *key,
-		GtkTreeIter *iter,
-		gpointer search_data)
+static void
+_threaded_app_list_init_func(gpointer user_data)
 {
-  gchar* app_name = NULL;
-  gtk_tree_model_get(model, iter,
-		     1, &app_name,
-		     -1);
-  if(app_name)
+  AppSearchHomePlugin* desktop_plugin = APP_SEARCH_HOME_PLUGIN(user_data);
+  GError* error = NULL;
+  desktop_plugin->priv->init_app_list_thread = 
+    g_thread_create(_fill_app_list_liststore_from_app_config_path,
+		    desktop_plugin, TRUE, &error);
+  if(error)
   {
-    gchar* key_downcase = g_utf8_strdown(key, -1);
-    gchar* app_name_downcase = g_utf8_strdown(app_name, -1);
-    gboolean key_app_name = strstr(app_name_downcase, key_downcase) != NULL;
-    g_free(key_downcase);
-    g_free(app_name_downcase);
-    g_free(app_name);
-    return !key_app_name;
+    g_warning("error on list init thread\n");
+    g_error_free(error);
   }
-  return TRUE;
+  else
+  {
+    return;
+  }
+  error = NULL;
+  GDir* directory = g_dir_open(CONFIG_PATH, 0, &error);
+  if(error)
+  {
+    g_warning("error on open directory %s\n", CONFIG_PATH);
+    g_error_free(error);
+    error = NULL;
+  }
+  else
+  {
+    const gchar* file_name = NULL;
+    while((file_name = g_dir_read_name(directory))!=NULL)
+    {
+      gchar* full_path = g_build_filename(CONFIG_PATH, file_name, NULL);
+      _add_app_from_desktop_file_to_liststore(full_path, desktop_plugin);
+      g_free(full_path);
+    }
+    g_dir_close(directory);
+  }
+  return;
 }
-*/
+
 static void
 _call_application(GtkTreeView       *tree_view,
 		  GtkTreePath       *path,
@@ -215,7 +347,7 @@ _call_application(GtkTreeView       *tree_view,
     gchar* app_desktop_file = NULL;
     gtk_tree_model_get(gtk_tree_view_get_model(tree_view),
 		       &iter,
-		       2, &app_desktop_file,
+		       4, &app_desktop_file,
 		       -1);
     if(app_desktop_file && g_str_has_suffix(app_desktop_file, ".desktop"))
     {
@@ -240,28 +372,34 @@ _filter_app_list_func(GtkTreeModel* model,
 		      GtkTreeIter* iter,
 		      gpointer data)
 {
-  g_return_if_fail(GTK_IS_ENTRY(data));
-  GtkEntry* entry = GTK_ENTRY(data);
+  
+  GtkEntry* entry = GTK_ENTRY(APP_SEARCH_HOME_PLUGIN(data)->priv->entry);
   
   if(gtk_entry_get_text_length(entry)>0)
   {
-    const gchar* text = gtk_entry_get_text(entry);
+    gchar* text = g_utf8_strdown(gtk_entry_get_text(entry),-1);
     gchar* app_name = NULL;
-    
+    gchar* app_comment = NULL;
     gtk_tree_model_get(model, iter,
-		       1, &app_name,
+		       2, &app_name,
+		       3, &app_comment,
 		       -1);
     gboolean visible = FALSE;
     if(app_name)
     {
       gchar* app_name_downcase = g_utf8_strdown(app_name, -1);
-      if(strstr(app_name_downcase, text)!=NULL)
+      gchar* app_comment_downcase = g_utf8_strdown(app_comment, -1);
+      if(strstr(app_name_downcase, text)!=NULL ||
+	 strstr(app_comment_downcase, text)!=NULL)
       {
 	visible = TRUE;
       }
       g_free(app_name);
       g_free(app_name_downcase);
+      g_free(app_comment);
+      g_free(app_comment_downcase);
     }
+    g_free(text);
     return visible;
   }
   
@@ -283,32 +421,19 @@ _button_release(GtkWidget* widget, GdkEventButton *event, AppSearchHomePlugin* d
   GtkWidget* dialog;
   GtkWidget* search_bar;
   GtkWidget *app_selector;
-  GtkListStore* app_list;
-  GtkTreeModel* filtered_app_list;
+  gtk_list_store_clear(desktop_plugin->priv->app_list);
 
-  dialog = gtk_dialog_new();/*_with_buttons("Start",
-				       NULL,
-				       (GtkDialogFlags)0,
-				       dgettext("hildon-libs","wdgt_bd_done"),
-				       GTK_RESPONSE_ACCEPT,
-				       NULL);*/
+  dialog = gtk_dialog_new();
   gtk_window_set_title(GTK_WINDOW(dialog), APP_TITLE);
   app_selector = hildon_gtk_tree_view_new(HILDON_UI_MODE_NORMAL);
   search_bar = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT);
+  desktop_plugin->priv->entry = search_bar;
   g_object_set(search_bar,"hildon-input-mode",
                HILDON_GTK_INPUT_MODE_FULL, NULL);
 
-  app_list = gtk_list_store_new(3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
-  _fill_app_list_liststore_from_app_config_path(app_list);
-  
-  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(app_list),
-				       1, GTK_SORT_ASCENDING);
-  filtered_app_list = gtk_tree_model_filter_new(GTK_TREE_MODEL(app_list),
-						NULL);
-  gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filtered_app_list),
-					 _filter_app_list_func,
-					 search_bar,
-					 NULL);
+
+  //  _fill_app_list_liststore_from_app_config_path(desktop_plugin);
+  _threaded_app_list_init_func(desktop_plugin);
 
   GtkTreeViewColumn* column_icon = gtk_tree_view_column_new();
   GtkTreeViewColumn* column_app_name = gtk_tree_view_column_new();
@@ -329,36 +454,25 @@ _button_release(GtkWidget* widget, GdkEventButton *event, AppSearchHomePlugin* d
 
   gtk_tree_view_column_set_attributes(column_app_name,
 				      app_name_renderer,
-				      "text",1,
+				      "markup",1,
 				      NULL);
 
   gtk_tree_view_set_model(GTK_TREE_VIEW(app_selector),
-			  GTK_TREE_MODEL(filtered_app_list));
+			  GTK_TREE_MODEL(desktop_plugin->priv->filtered_app_list));
   
 
-  /*
-  gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(app_selector),
-				      search_app_func,
-				      desktop_plugin,
-				      NULL);
-  */
-  /*
-  gtk_tree_view_set_search_entry(GTK_TREE_VIEW(app_selector),
-				 GTK_ENTRY(search_bar));
-  */
   GtkWidget* pan = hildon_pannable_area_new();
   gtk_container_add(GTK_CONTAINER(pan), app_selector);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), pan, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), search_bar,FALSE,FALSE,0); 
   g_signal_connect(app_selector, "row-activated", G_CALLBACK(_call_application), dialog);
-  g_signal_connect(search_bar, "changed", G_CALLBACK(_refilter_by_text), filtered_app_list);
-  gtk_window_set_default_size(GTK_WINDOW(dialog),-1,250);
+  g_signal_connect(search_bar, "changed", G_CALLBACK(_refilter_by_text), desktop_plugin->priv->filtered_app_list);
+  gtk_window_set_default_size(GTK_WINDOW(dialog), -1, 350);
   gtk_widget_grab_focus(search_bar);
   gtk_widget_show_all(dialog);
   gtk_dialog_run(GTK_DIALOG(dialog));
+  
   gtk_widget_destroy(dialog);
-  g_object_unref(app_list);
-  g_object_unref(filtered_app_list);
 }
 
 
@@ -394,11 +508,29 @@ app_search_finalize(GObject* object)
 {
   AppSearchHomePlugin* desktop_plugin = APP_SEARCH_HOME_PLUGIN(object);
   G_OBJECT_CLASS(app_search_home_plugin_parent_class)->finalize(object);
+  g_object_unref(desktop_plugin->priv->app_list);
+  g_object_unref(desktop_plugin->priv->filtered_app_list);
+
 }
 
 static void
 app_search_home_plugin_init(AppSearchHomePlugin* desktop_plugin)
 {
+  desktop_plugin->priv = APP_SEARCH_HOME_PLUGIN_GET_PRIVATE(desktop_plugin);
+  desktop_plugin->priv->app_list = gtk_list_store_new(5, GDK_TYPE_PIXBUF, G_TYPE_STRING, 
+						      G_TYPE_STRING, 
+						      G_TYPE_STRING, 
+						      G_TYPE_STRING);
+  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(desktop_plugin->priv->app_list),
+				       1, GTK_SORT_ASCENDING);
+  desktop_plugin->priv->filtered_app_list = gtk_tree_model_filter_new(GTK_TREE_MODEL(desktop_plugin->priv->app_list),
+								      NULL);
+
+  gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(desktop_plugin->priv->filtered_app_list),
+					 _filter_app_list_func,
+					 desktop_plugin,
+					 NULL);
+
   GtkWidget *contents = gtk_event_box_new();
   gtk_event_box_set_visible_window(GTK_EVENT_BOX(contents), FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(contents), 0);
@@ -416,6 +548,7 @@ static void
 app_search_home_plugin_class_init(AppSearchHomePluginClass* klass)
 {
   GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(klass);
+  g_type_class_add_private(klass, sizeof(AppSearchHomePluginPrivate));
   widget_class->realize = app_search_realize;
   widget_class->expose_event = app_search_expose;
   G_OBJECT_CLASS(klass)->finalize = app_search_finalize;
