@@ -17,9 +17,11 @@ typedef struct _AppSearchHomePluginClass AppSearchHomePluginClass;
 #define APP_SEARCH_IS_HOME_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APP_SEARCH_TYPE_HOME_PLUGIN))
 #define APP_SEARCH_HOME_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APP_SEARCH_TYPE_HOME_PLUGIN, AppSearchHomePluginClass))
 
+typedef struct _AppSearchHomePluginPrivate AppSearchHomePluginPrivate;
 struct _AppSearchHomePlugin
 {
   HDHomePluginItem hitem;
+  AppSearchHomePluginPrivate* priv;
 };
 
 struct _AppSearchHomePluginClass
